/* filename: asymp_ci.g
** created:  06/18/03
** Modifications:
**
*/

new;
cls;
library pgraph, user;
_fcmptol = 1E-10;

#include nkmp_spec.g
#include pathspec.g;

declare matrix gmat_global, lasel_global;

/******************************************************** 
** Configuration of Monte Carlo
*/

mcrun    = "1";
nblocks  = 100;
nsim     = 10000;
__output = 0;

alpha_u  = 0.05;
alpha_s  = 0.05;
zcrit    = cdfni(1-(alpha_u+alpha_s)/2);

/* Load model parameters
*/
lpath          = resupath;
ljlim          = lpath $+ "\\jlim";
lqlim          = lpath $+ "\\qlim";
lnu            = lpath $+ "\\nu";

open fhjlim    = ^ljlim for read;
open fhqlim    = ^lqlim for read;
open fhnu      = ^lnu for read;

JJ  = readr( fhjlim, hh); 
JJc = chol(JJ)';
QQ  = readr( fhqlim, npara);
nu  = readr( fhnu,hh);
nu  = nu[h1+1:hh];

closeall fhjlim, fhqlim, fhnu;

MM =  zeros(h2,h1) ~ ones(h2,h2);

J11 = JJ[1:h1,1:h1];
J12 = JJ[1:h1,h1+1:hh];
J21 = JJ[h1+1:hh,1:h1];
J22 = JJ[h1+1:hh,h1+1:hh];

J211 = J22-J21*inv(J11)*J12;

QQ1  = QQ[.,1:h1];
QQ2  = QQ[.,h1+1:hh];

shat1_cov = inv(QQ1*inv(J11)*QQ1');
shat2_cov = inv(QQ*inv(JJ)*QQ');

uhat1_cov = J22 + QQ2'*inv(QQ1*inv(J11)*QQ1')*QQ2
          - 2*QQ2'*inv(QQ1*inv(J11)*QQ1')*QQ1*inv(J11)*J12;

RR  = -QQ | MM;
QQD       = DD*QQ;
RRD       = -QQD | MM;

RJRRJ = inv(RR*inv(JJ)*RR')*RR*inv(JJ);
QJQQJ = inv(QQ*inv(JJ)*QQ')*QQ*inv(JJ);
QJQQJ_D   = inv(QQD*inv(JJ)*QQD')*QQD*inv(JJ);
RJRRJ_D   = inv(RRD*inv(JJ)*RRD')*RRD*inv(JJ);

JJinv = inv(JJ);    
J_JQQJQQJ = inv(JJ) - inv(JJ)*QQ'*inv(QQ*inv(JJ)*QQ')*QQ*inv(JJ); 

"covariance of shat1";
shat1_cov;

"";
"covariance of shat2";
shat2_cov;

"";
"covariance of uhat1";
uhat1_cov;

"";
"Press Key to Continue";
kpress = keyw;

/* u for DGP
*/
Tsim_dgp = Tsim_grid[subT];
u_dgp  = nu*sqrt(Tsim_dgp);

nsim_crit = 100000;

/* Grid for u_test
*/
u_test_min  = 0;
u_test_max  = u_dgp+5*sqrt(uhat1_cov);
u_test_n    = 51;
u_test_step = (u_test_max-u_test_min)/(u_test_n-1);
u_test_grid = seqa(u_test_min,u_test_step,u_test_n);

/* s-grid (uncentered)
*/
s_min  = 0;
s_max  = 3*sqrt(shat1_cov[parasel,parasel]);
s_n    = 301;
s_step = (s_max-s_min)/(s_n-1);
s_griduncentered = seqa(s_min,s_step,s_n);

/******************************************************** 
** Initialize Output files
*/

nparaout  = nparaest+hh+1;

opath  = asymppath $+ "\\mcrun" $+ mcrun;
chkdir = chk_dir(opath);

ociu0s   = opath $+ "\\" $+ paraselstr $+ subTstr $+ "ciu0_s";
ocis0s   = opath $+ "\\" $+ paraselstr $+ subTstr $+ "cis0_s";
ocis1s   = opath $+ "\\" $+ paraselstr $+ subTstr $+ "cis1_s";

create fhciu0s  =^ociu0s with SUM, 4, 8; /* true value incl?,length,min,max */
create fhcis0s  =^ocis0s with SUM, 4, 8; /* true value incl?,length,min,max */
create fhcis1s  =^ocis1s with SUM, 4, 8; /* true value incl?,length,min,max */

/******************************************************** 
** Monte Carlo Iterations
*/

/* compute critical values
*/
tstart = date;

critval_u  = crit_u(JJ,QQ,u_test_grid,alpha_u,nsim_crit);
critval_s  = crit_s(JJ,QQ,DD,u_test_grid,alpha_s,nsim_crit);

"Max / last critical value for u:" maxc(critval_u) critval_u[rows(critval_u)];
"Max / last critical value for s:" maxc(critval_s) critval_s[rows(critval_u)];
"Chi2 critval 2.71 / 3.84";
tend = date;
"Elapsed Time"; etstr(ethsec(tstart,tend));
"--------------------------------------------------------------------";
"";
pause(1e-5);

iblock = 1;
do until iblock > nblocks;

   tstart = date;
   
   ciu0ssim     = zeros(nsim,4);
   cis0ssim     = zeros(nsim,4);
   cis1ssim     = zeros(nsim,4);

   j = 1;
   
   do until j > nsim;

     /* Fix the shocks
     */
     Z = JJc*rndn(hh,1);
     Z1 = Z[1:h1,.];
     Z2 = Z[h1+1:hh,.];

     /* Compute transformations of Z, partitions of Q, J
     */           
     Ztil   = Z + MM'*u_dgp;
     Ztil1  = Ztil[1:h1,.];
     Ztil2  = Ztil[h1+1:hh,.];
          
     ciu0 = zeros(1,u_test_n);
     cis0 = zeros(1,2);
     
     /* Version 1: use equality conditions only
     */
     shat1   = -inv(QQ1*inv(J11)*QQ1')*QQ1*inv(J11)*Ztil1;
     cis1min = shat1[parasel] - zcrit*sqrt(shat1_cov[parasel,parasel]);
     cis1max = shat1[parasel] + zcrit*sqrt(shat1_cov[parasel,parasel]);
     cis1length = 2*zcrit*sqrt(shat1_cov[parasel,parasel]);
     cis1cov = (0 ge cis1min) and (0 le cis1max);

     cis1ssim[j,.] = cis1cov~cis1length~cis1min~cis1max;


     /* Version 0: use equality and inequality conditions 
     ** unrestricted min
     */

     phihat0 =  RJRRJ*Ztil;
         
     if phihat0[nparaest+1:nparaest+h2,.] < 0;
          phihat0[1:nparaest,.] = -QJQQJ*Ztil;
          phihat0[nparaest+1:nparaest+h2,.] = zeros(h2,1);
     endif;

     elfcn0 = 0.5*(Ztil - RR'*phihat0)'*JJinv*(Ztil - RR'*phihat0);
   
     /* confidence interval for u
     */

     if alpha_u == 0;

        supcritval_s = maxc(critval_s);
        ciu0 = ones(1,u_test_n);

     else;

        supcritval_s = 0;

        u_ind = 1;
        do until u_ind > u_test_n;     
     
          u_test = u_test_grid[u_ind];

          /* Version 12: impose that u=u0
          */            
          elfcn12 = 0.5*(Ztil-MM'*u_test)'*J_JQQJQQJ*(Ztil-MM'*u_test);           

          /* Compute Test Statistic for u=u0
          ** if not rejected, include point in confidence interval
          */
          lrtest1_0  = 2*( elfcn12 - elfcn0 );        
          ciu0[1,u_ind] = lrtest1_0 < critval_u[u_ind,1];

          if ciu0[1,u_ind] == 1;
             supcritval_s = maxc(supcritval_s|critval_s[u_ind]);
          endif;

          u_ind = u_ind + 1;

        endo;     

     endif;

     /* compute the length of the u interval
     */
     ciu0length = (sumc(ciu0')-1)*u_test_step;
     ciu0lim = ciu0'.*u_test_grid;
     ciu0lim = delif(ciu0lim,(ciu0'.==0));
     ciu0min = minc(ciu0lim);
     ciu0max = maxc(ciu0lim);
     ciu0cov = (u_dgp ge ciu0min) and (u_dgp le ciu0max);
     ciu0ssim[j,.] = ciu0cov ~ ciu0length~ciu0min~ciu0max;

     /* compute confidence interval for s
     */
     s_gridmin = phihat0[parasel]-s_max;
     s_gridmax = phihat0[parasel]+s_max;

     
     /* lower bound for confidence interval */
     found = 0;
     s_test = zeros(nparaest,1);
     s_test[parasel,1] = phihat0[parasel];

     do until found == 1;
         
          Ztiltil = Ztil + QQ'*s_test;

          phihat00 =  RJRRJ_D*Ztiltil;
                      
          if phihat00[rows(DD)+1:rows(DD)+h2,.] < 0;
                phihat00[1:rows(DD),.] = -QJQQJ_D*Ztiltil;
                phihat00[rows(DD)+1:rows(DD)+h2,.] = zeros(h2,1);
          endif;

          elfcn00 = 0.5*(Ztiltil - RRD'*phihat00)'*JJinv*(Ztiltil - RRD'*phihat00);
          found = 2*(elfcn00-elfcn0) > supcritval_s;
          s_test[parasel,1] = s_test[parasel,1]-s_step;
 
     endo;

     cis0min = s_test[parasel,1]+s_step;

     /* upper bound for confidence interval */
     found = 0;
     s_test = zeros(nparaest,1);
     s_test[parasel,1] = phihat0[parasel];

     do until found == 1;
                      
          Ztiltil = Ztil + QQ'*s_test;

          phihat00 =  RJRRJ_D*Ztiltil;
                      
          if phihat00[rows(DD)+1:rows(DD)+h2,.] < 0;
                phihat00[1:rows(DD),.] = -QJQQJ_D*Ztiltil;
                phihat00[rows(DD)+1:rows(DD)+h2,.] = zeros(h2,1);
          endif;

          elfcn00 = 0.5*(Ztiltil - RRD'*phihat00)'*JJinv*(Ztiltil - RRD'*phihat00);
          found = 2*(elfcn00-elfcn0) > supcritval_s;
          s_test[parasel,1] = s_test[parasel,1]+s_step;
 
     endo;

     cis0max = s_test[parasel,1]-s_step;
     
     /* compute length of the s interval, taking union of intervals
     */
     cis0length = cis0max - cis0min;
     cis0cov = (0 ge cis0min) and (0 le cis0max);
     cis0ssim[j,.] = cis0cov ~ cis0length~cis0min~cis0max;

     /*
     "u interv bds" u_test_min u_test_max;
     "u interv    " ciu0ssim[j,3 4 2 1];
     "";
     "s interv bds" s_gridmin s_gridmax;
     "s interv    " cis0ssim[j,3 4 2 1];
     "s1 interv l " 2*zcrit*sqrt(shat1_cov[parasel,parasel]);
     "------------------------------------";
     kentry = keyw;
     */

     j = j+1;

    endo;   

    "Block   " iblock "of" nblocks;
    
    wr = writer(fhciu0s,ciu0ssim);
    wr = writer(fhcis0s,cis0ssim);
    wr = writer(fhcis1s,cis1ssim);
     
    tend = date;
    "Elapsed Time"; etstr(ethsec(tstart,tend));
    pause(1e-5);

    iblock = iblock+1;
endo;  

closeall fhcis0s, fhciu0s, fhcis1s;     
     
end;

#include critval.src
