/* filename: asymp_sim.g
** created:  06/18/03
** Modifications:
**
*/

new;
cls;
library pgraph, user;
_fcmptol = 1E-10;

#include nkmp_spec.g
#include pathspec.g;

declare matrix gmat_global, lasel_global;

/******************************************************** 
** Configuration of Monte Carlo
*/

mcrun    = "1";
nblocks  = 100;
nsim     = 10000;
__output = 0;
nstat    = 2;


/* Load model parameters
*/
lpath          = resupath;
ljlim          = lpath $+ "\\jlim";
lqlim          = lpath $+ "\\qlim";
lnu            = lpath $+ "\\nu";

open fhjlim    = ^ljlim for read;
open fhqlim    = ^lqlim for read;
open fhnu      = ^lnu for read;

JJ  = readr( fhjlim, hh); 
JJc = chol(JJ)';
QQ  = readr( fhqlim, npara);
nu  = readr( fhnu,hh);
nu  = nu[h1+1:hh];

closeall fhjlim, fhqlim, fhnu;

MM =  zeros(h2,h1) ~ ones(h2,h2);

J11 = JJ[1:h1,1:h1];
J12 = JJ[1:h1,h1+1:hh];
J21 = JJ[h1+1:hh,1:h1];
J22 = JJ[h1+1:hh,h1+1:hh];

J211 = J22-J21*inv(J11)*J12;

QQ1  = QQ[.,1:h1];
QQ2  = QQ[.,h1+1:hh];

s1hat_cov = inv(QQ1*inv(J11)*QQ1');
s2hat_cov = inv(QQ*inv(JJ)*QQ');

"covariance of s1hat";
s1hat_cov;

"covariance of s2hat";
s2hat_cov;

/* Grid for  u
*/
u_grid = sqrt(Tsim_grid) ;
u_grid = nu*u_grid;
u_n    = rows(u_grid);

/******************************************************** 
** Initialize Output files
*/

nparaout  = nparaest+hh+1;

opath  = asymppath $+ "\\mcrun" $+ mcrun;
chkdir = chk_dir(opath);

opara0  = opath $+ "\\0pa";
opara1  = opath $+ "\\1pa";
opara12 = opath $+ "\\12pa";

create fhpara0 =^opara0 with PARA,  u_n*nparaout, 8;
create fhpara1 =^opara1 with PARA,  u_n*nparaout, 8;
create fhpara12=^opara12 with PARA, u_n*nparaout, 8;

/******************************************************** 
** Monte Carlo Iterations
*/

iblock = 1;
do until iblock > nblocks;

   tstart = date;

   para0sim  = zeros(nsim,u_n*nparaout);
   para1sim  = zeros(nsim,u_n*nparaout);
   para12sim = zeros(nsim,u_n*nparaout);
  
   j = 1;
   
   do until j > nsim;

     /* Fix the shocks
     */
     Z = JJc*rndn(hh,1);
     Z1 = Z[1:h1,.];
     Z2 = Z[h1+1:hh,.];

     /* Iterate over u
     */
     u_ind = 1;
     do until u_ind > u_n;

           paraind = u_ind;
           u = u_grid[u_ind];

           /* Compute transformations of Z, partitions of Q, J
           */           

           Ztil   = Z + MM'*u;
           Ztil1  = Ztil[1:h1,.];
           Ztil2  = Ztil[h1+1:hh,.];
           
           RR  = -QQ | MM;

           /* Version 0: use equality and inequality conditions 
           */

           phihat0 =  inv(RR*inv(JJ)*RR')*RR*inv(JJ)*Ztil;
         
           if phihat0[nparaest+1:nparaest+h2,.] < 0;
              phihat0[1:nparaest,.] = -inv(QQ*inv(JJ)*QQ')*QQ*inv(JJ)*Ztil;
              phihat0[nparaest+1:nparaest+h2,.] = zeros(h2,1);
           endif;

           shat0 = phihat0[1:nparaest,.];
           lahat0 = inv(JJ)*(Ztil-RR'*phihat0);
           elfcn0 = 0.5*(Ztil - RR'*phihat0)'*inv(JJ)*(Ztil - RR'*phihat0);
        
           para0sim[j,(paraind-1)*nparaout+1:paraind*nparaout] 
              =  shat0' ~ lahat0' ~ elfcn0;
                       
           /* Version 1: use only equality conditions 
           */

           shat1 = -inv(QQ1*inv(J11)*QQ1')*QQ1*inv(J11)*Ztil1;
           lahat1 = inv(J11)*(Ztil1+QQ1'*shat1) | 0;
           elfcn1 = 0.5*Ztil1'*(inv(J11) - inv(J11)*QQ1'*inv(QQ1*inv(J11)*QQ1')*QQ1*inv(J11))*Ztil1;           

           para1sim[j,(paraind-1)*nparaout+1:paraind*nparaout] 
              =  shat1' ~ lahat1' ~ elfcn1;
                 
           /* Version 12: treat both sets of conditions as equalities
           */

           shat12 = -inv(QQ*inv(JJ)*QQ')*QQ*inv(JJ)*Ztil;
           lahat12 = inv(JJ)*(Ztil+QQ'*shat12);
           elfcn12 = 0.5*Ztil'*(inv(JJ) - inv(JJ)*QQ'*inv(QQ*inv(JJ)*QQ')*QQ*inv(JJ))*Ztil;           

           para12sim[j,(paraind-1)*nparaout+1:paraind*nparaout] 
              =  shat12'  ~ lahat12' ~ elfcn12;
                 
           u_ind = u_ind + 1;

     endo;
    
     j = j+1;

    endo;

    "Block   " iblock "of" nblocks;
    pause(1e-5);

    wr = writer(fhpara0,para0sim);
    wr = writer(fhpara1,para1sim);
    wr = writer(fhpara12,para12sim);

    tend = date;
    "Elapsed Time"; etstr(ethsec(tstart,tend));

    iblock = iblock+1;
endo;  

closeall fhpara0, fhpara1, fhpara12;
end;
