/* filename: mc_sim.g
** created:  06/18/03
** Modifications:
**
*/

new;
cls;
library pgraph, user;
_fcmptol = 1E-10;

#include nkmp_spec.g
#include pathspec.g;
#include nkmp_mod.src
#include nkmp_para.src
#include nkmp_sim.src

declare matrix gmat_global, lasel_global;


/******************************************************** 
** Configuration of Monte Carlo
*/

/*
rndseed(1799);
*/

mcrun    = "4";
nblocks  = 1;
nsim     = 1000;
__output = 0;
nstat    = 2;

alpha_u  = 0.00;
alpha_s  = 0.10;
zcrit    = cdfni(1-(alpha_u+alpha_s)/2);

/* Load model parameters
*/
load path = ^dgppath para[] = "dgp_para.out";
para = reshape(para,rows(para)/ndgp,ndgp);
para      = para[.,dgp];

th_true = para[5] | (1-para[5])*para[3] | 4*(1-para[5])*para[4];

lpath          = resupath;
ljlim          = lpath $+ "\\jlim";
lqlim          = lpath $+ "\\qlim";
lnu            = lpath $+ "\\nu";

open fhjlim    = ^ljlim for read;
open fhqlim    = ^lqlim for read;
open fhnu      = ^lnu for read;

JJ  = readr( fhjlim, hh); 
JJc = chol(JJ)';
QQ  = readr( fhqlim, npara);
nu  = readr( fhnu,hh);
nu  = nu[h1+1:hh];

closeall fhjlim, fhqlim, fhnu;

MM =  zeros(h2,h1) ~ ones(h2,h2);

J11 = JJ[1:h1,1:h1];
J12 = JJ[1:h1,h1+1:hh];
J21 = JJ[h1+1:hh,1:h1];
J22 = JJ[h1+1:hh,h1+1:hh];

J211 = J22-J21*inv(J11)*J12;

QQ1  = QQ[.,1:h1];
QQ2  = QQ[.,h1+1:hh];

shat1_cov = inv(QQ1*inv(J11)*QQ1');
shat2_cov = inv(QQ*inv(JJ)*QQ');

uhat1_cov = J22 + QQ2'*inv(QQ1*inv(J11)*QQ1')*QQ2
          - 2*QQ2'*inv(QQ1*inv(J11)*QQ1')*QQ1*inv(J11)*J12;

RR  = -QQ | MM;
QQD       = DD*QQ;
RRD       = -QQD | MM;

RJRRJ = inv(RR*inv(JJ)*RR')*RR*inv(JJ);
QJQQJ = inv(QQ*inv(JJ)*QQ')*QQ*inv(JJ);
QJQQJ_D   = inv(QQD*inv(JJ)*QQD')*QQD*inv(JJ);
RJRRJ_D   = inv(RRD*inv(JJ)*RRD')*RRD*inv(JJ);

JJinv = inv(JJ);    
J_JQQJQQJ = inv(JJ) - inv(JJ)*QQ'*inv(QQ*inv(JJ)*QQ')*QQ*inv(JJ); 

"Significance levels: "  "alpha_u is " alpha_u ",  alpha_s is " alpha_s;

"covariance of shat1";
shat1_cov;

"";
"covariance of shat2";
shat2_cov;

"";
"covariance of uhat1";
uhat1_cov;

"";
"Press Key to Continue";
kpress = keyw;

Tsim = Tsim_grid[subT];      

/* Grid for u_test
*/
u_test_n    = 101;

/* s-grid (uncentered)
*/
s_min  = 0; 
s_max  = 5*sqrt(shat1_cov[parasel,parasel]);
s_n    = 21;
s_step = (s_max-s_min)/(s_n-1);
s_griduncentered = seqa(s_min,s_step,s_n);
s_itmax = 1000;


/* Starting values for hessian
*/
H0  = eye(nparaest);

/******************************************************** 
** Initialize Output files
*/

nparaout  = nparaest+hh+1;
opath     = mcpath $+ "\\mcrun" $+ mcrun;
chkdir = chk_dir(opath);

ociu0s  = opath $+ "\\" $+ paraselstr $+ subTstr $+ "ciu0_s";
ocis0s  = opath $+ "\\" $+ paraselstr $+ subTstr $+ "cis0_s";
ocis1s  = opath $+ "\\" $+ paraselstr $+ subTstr $+ "cis1_s";

create fhciu0s =^ociu0s with SUM, 4, 8; /* true value incl?,length,min,max */
create fhcis0s =^ocis0s with SUM, 4, 8; /* true value incl?,length,min,max */
create fhcis1s =^ocis1s with SUM, 4, 8; /* true value incl?,length,min,max */

/** start log file **/
outstr = opath $+ "\\mc_ci" $+ paraselstr $+ subTstr $+ ".log";
output file = ^outstr reset;

/******************************************************** 
** Monte Carlo Iterations
*/

error_smin = 0;
error_smax = 0;
error_thhat0 = 0;

iblock = 1;
do until iblock > nblocks;

   tstart = date;

   ciu0ssim    = zeros(nsim,4);
   cis0ssim    = zeros(nsim,4);
   cis1ssim    = zeros(nsim,4);

   j = 1;
   
   do until j > nsim;

           startloop:

           /* Generate Data
           */             
           yt = dsgesim(para,msel,Tsimburn,Tsimtotal);
           yt = yt - meanc(yt)';
           /* yt = Ygr, pi, R */


           /* Grid for u_test
           */
           u_dgp       = nu*sqrt(Tsim);
           u_test_min  = maxc(0| (u_dgp-5*sqrt(uhat1_cov)));
           u_test_max  = u_dgp+5*sqrt(uhat1_cov);
           u_test_step = (u_test_max-u_test_min)/(u_test_n-1); 
           u_test_grid = seqa(u_test_min,u_test_step,u_test_n);

           XY = yt[2:Tsim+1,3]; 
           /* R */
           
           XX = yt[1:Tsim,3] ~ yt[2:Tsim+1,2] ~ yt[2:Tsim+1,1];
           /* R(t-1), pi(t), Ygr(t) */
           
           X1 = yt[1:Tsim,3] ~ yt[1:Tsim,2] ~ yt[1:Tsim,1];
           /* R(t-1), pi(t-1), Ygr(t-1) */
           
           X2 = -yt[2:Tsim+1,1] ;
           /* -Ygr(t) */

           X12 = X1~X2;


           if (j==1);

             sighat = sqrt((XY - XX*th_true)'*(XY - XX*th_true)/Tsim);
             nuhat  = (XY - XX*th_true)'*X2/Tsim;
             thhat1 = (X1'*XY)/(X1'*XX);
  
             Qhat = -XX'*X12/Tsim;
             Jhat = X12'*X12/Tsim;
             Jhat = Jhat*sighat^2; 

             "Verification";
             "QQ / Qhat";
             QQ; Qhat;
             "JJ / Jhat";
             JJ; Jhat;
             "nu / nuhat";
             nu; nuhat;
             "sighat";
             sighat;
             "Estimated / True Parameters";
             thhat1'; th_true';
             "";
             "";
             "=================================================================";
 
           endif;
              
           Xmat = XY ~ XX ~ X1 ~ X2;
           retcode_cum = 0;
                             
         
           /* Version 1: use only equality conditions -> IV estimator
           */

           thhat1  = (X1'*XY)/(X1'*XX);  
           sighat1 = sqrt((XY - XX*thhat1)'*(XY - XX*thhat1)/Tsim); 
           shat1   = sqrt(Tsim)*(thhat1-th_true);

           Qhat1 = -XX'*X1/Tsim;
           Jhat1 =  X1'*X1/Tsim;
           Jhat1 =  Jhat1*sighat1^2;

           shat1_sehat = sqrt(diag(inv(Qhat1*inv(Jhat1)*Qhat1')));
           shat1_sehat = shat1_sehat[parasel];           

           cis1min  = shat1[parasel] - zcrit*shat1_sehat;
           cis1max  = shat1[parasel] + zcrit*shat1_sehat;
           cis1length = 2*zcrit*shat1_sehat;
           cis1cov  = (0 ge cis1min) and (0 le cis1max);

           cis1ssim[j,.] = cis1cov~cis1length~cis1min~cis1max;
         
           /* Version 0: use equality and inequality condition
           */

           ciu0 = zeros(1,u_test_n);
           cis0 = zeros(1,2);

           { elfcn0, thhat0, lahat0, g, H, itct, fcount, retcode0 } 
           = csmin_el(thhat1,H0,crit_global,nit_global,&objfcn_th,&grad_th);
           
          
           if retcode0 > 0;

              { elfcn0, thhat0, lahat0, g, H, itct, fcount, retcode0 } 
              = csmin_el(th_true,H0,crit_global,nit_global,&objfcn_th,&grad_th);

           endif;

           if retcode0 > 0;
              "Baseline estimation failed - generate new data";
              goto startloop;
           endif;

           
           { elfcn0, lahat0, phat0, retcode0 } = objfcn_th(thhat0);
           shat0   = sqrt(Tsim)*(thhat0-th_true);  

           "Iteration   " j Tsim;
           "--------------------------------------------------------";
           "";
           "Baseline Estimate - thhat0";
           "true   " th_true';
           "thhat0 " thhat0';
           pause(1E-5);        

           /* Calculate Estimates of Q and J
           */
           sighat0 = sqrt((XY - XX*thhat0)'*(XY - XX*thhat0)/Tsim);         

           Qhat = -XX'*X12/Tsim;
           Jhat =  X12'*X12/Tsim;
           Jhat =  Jhat*sighat0^2;

           /* Confidence Interval for u
           */

           if alpha_u == 0;

              "Calculating Critical Values";
              "Calculating Confidence Interval for u";
              pause(1e-5);

              ciu0 = ones(1,u_test_n);
              supcritval_s = maxc(crit_s(Jhat,Qhat,DD,0,alpha_s,100000)|zcrit^2);

           else;

              /* Calculate Critical Values
              */
              "Calculating Critical Values";
              pause(1e-5);
              critval_u  = crit_u(Jhat,Qhat,u_test_grid,alpha_u,100000);
              critval_s  = crit_s(Jhat,Qhat,DD,u_test_grid,alpha_s,100000);
              supcritval_s = 0;

              "Calculating Confidence Interval for u";
              pause(1e-5);

              nu_global = 0;

              thhat12_gmm = inv(XX'*X12*inv(X12'*X12)*X12'*XX)*XX'*X12*inv(X12'*X12)*X12'*XY;

              { elfcn12, thhat12_old, lahat12, g, H, itct, fcount, retcode12 } 
              = csmin_el(th_true,H0,crit_global,nit_global,&objfcn_th12,&grad_th12);

              if retcode12 > 0;
                 { elfcn12, thhat12_old, lahat12, g, H, itct, fcount, retcode12 } 
                 = csmin_el(thhat12_gmm,H0,crit_global,nit_global,&objfcn_th12,&grad_th12);
              endif;

              if retcode12 > 0;
                 "Problems with Thetahat12 - Generate new data";
                 goto startloop;
              endif;

              
              u_ind = 1;
              do until u_ind > u_test_n;     
     
                u_test = u_test_grid[u_ind];

                /* Version 12: impose that u=u0
                */
                nu_global = u_test/sqrt(Tsim);

                { elfcn12, thhat12, lahat12, g, H, itct, fcount, retcode12 } 
                = csmin_el(thhat12_old,H0,crit_global,nit_global,&objfcn_th12,&grad_th12);
                retcode_cum = retcode_cum + retcode12;

                { elfcn12, lahat12, phat12, retcode12 } = objfcn_th12(thhat12);
                thhat12_old = thhat12;

                lrtest = 2*Tsim*( elfcn12 - elfcn0 );        
                ciu0[1,u_ind] = lrtest < critval_u[u_ind,1];
 
                /*
                "u-interval" u_test~lrtest~critval_u[u_ind,1]~retcode12;
                pause(1e-5);
                */

                if ciu0[1,u_ind] == 1;
                  supcritval_s = maxc(supcritval_s|critval_s[u_ind]);
                endif;

                u_ind = u_ind+1;

              endo;   

           endif;

           /* compute the length of the u interval
           */
           ciu0length = (sumc(ciu0')-1)*u_test_step;
           ciu0lim = ciu0'.*u_test_grid;
           ciu0lim = delif(ciu0lim,(ciu0'.==0));
           ciu0min = minc(ciu0lim);
           ciu0max = maxc(ciu0lim);
           ciu0cov = (nu ge ciu0min) and (nu le ciu0max);
           ciu0ssim[j,.] = ciu0cov ~ ciu0length ~ ciu0min ~ ciu0max;

           /* Confidence Interval for s
           */

           /* upper bound */
           "Calculating Confidence Interval for s - upper bound";
           pause(1e-5);
                
           thhat00_old = DD*thhat0;
           s_test = shat0[parasel];

           thfix_global = th_true[parasel] + s_test/sqrt(Tsim);
           DD_global = DD;
           parasel_global = parasel;

           { elfcn00, thhat00, lahat00, g, H, itct, fcount, retcode0 } 
           = csmin_el(thhat00_old,eye(nparaest-1),crit_global,nit_global,&objfcn_thdd,&grad_thdd);

           if (elfcn00-elfcn0)^2 > 1E-10;
              "Problems with thhat0";
              error_thhat0 = error_thhat0+1;
              goto startloop;
           endif;

           found = 0;
           s_ind = 1;
           do until found == 1;

                    if s_ind > s_itmax;
                       "Terminate s-high iterations";
                       error_smax = error_smax + 1;
                       goto startloop;
                    endif;
                                 
                    thfix_global = th_true[parasel] + s_test/sqrt(Tsim);
                    DD_global = DD;
                    parasel_global = parasel;

                    { elfcn00, thhat00, lahat00, g, H, itct, fcount, retcode0 } 
                    = csmin_el(thhat00_old,eye(nparaest-1),crit_global,nit_global,&objfcn_thdd,&grad_thdd);
                    retcode_cum = retcode_cum + retcode0;

                    { elfcn00, lahat00, phat00, retcode00 } = objfcn_thdd(thhat00);
                    thhat00_old = thhat00;

                    lrtest = 2*Tsim*(elfcn00-elfcn0); 
                    found  = lrtest > supcritval_s;
                   
                    
                    "s-loop high" s_test~lrtest~supcritval_s;
                    pause(1e-5);
                    
                    
                    s_test = s_test + s_step;
                    s_ind = s_ind+1;

     
           endo;

           s_test = s_test - s_step;

           found = 0;
           s_ind = 1;
           do until found == 1;

                    if s_ind > s_itmax;
                       "Terminate s-high iterations";
                       error_smax = error_smax + 1;
                       goto startloop;
                    endif;
                                 
                    thfix_global = th_true[parasel] + s_test/sqrt(Tsim);
                    DD_global = DD;
                    parasel_global = parasel;

                    { elfcn00, thhat00, lahat00, g, H, itct, fcount, retcode0 } 
                    = csmin_el(thhat00_old,eye(nparaest-1),crit_global,nit_global,&objfcn_thdd,&grad_thdd);
                    retcode_cum = retcode_cum + retcode0;

                    { elfcn00, lahat00, phat00, retcode00 } = objfcn_thdd(thhat00);
                    thhat00_old = thhat00;

                    lrtest = 2*Tsim*(elfcn00-elfcn0); 
                    found  = lrtest < supcritval_s;
                   
                    
                    "s-loop high - reverse" s_test~lrtest~supcritval_s;
                    pause(1e-5);
                    
                    
                    s_test = s_test - s_step/s_n;
                    s_ind = s_ind+1;

           endo;

           cis0max = s_test + s_step/s_n;

           /* lower bound */
           "Calculating Confidence Interval for s - lower bound";
           pause(1e-5);

           thhat00_old = DD*thhat0;
           s_test = shat0[parasel];

           found = 0;
           s_ind = 1;

           do until found == 1;

                    if s_ind > s_itmax;
                       "Terminate s-low iterations";
                       error_smin = error_smin + 1;
                       goto startloop;
                    endif;
                    
                    thfix_global = th_true[parasel] + s_test/sqrt(Tsim);

                    DD_global = DD;
                    parasel_global = parasel;

                    { elfcn00, thhat00, lahat00, g, H, itct, fcount, retcode0 } 
                    = csmin_el(thhat00_old,eye(nparaest-1),crit_global,nit_global,&objfcn_thdd,&grad_thdd);
                    retcode_cum = retcode_cum + retcode0;

                    { elfcn00, lahat00, phat00, retcode00 } = objfcn_thdd(thhat00);
                    thhat00_old = thhat00;

                    lrtest = 2*Tsim*(elfcn00-elfcn0); 
                    found  = lrtest > supcritval_s;
                   
                    
                    "s-loop low" s_test~lrtest~supcritval_s;
                    pause(1e-5);
                    

                    s_test = s_test - s_step;
                    s_ind = s_ind+1;
                    
           endo;

           s_test = s_test + s_step;

           found = 0;
           s_ind = 1;

           do until found == 1;

                    if s_ind > s_itmax;
                       "Terminate s-low iterations";
                       error_smin = error_smin + 1;
                       goto startloop;
                    endif;
                    
                    thfix_global = th_true[parasel] + s_test/sqrt(Tsim);

                    DD_global = DD;
                    parasel_global = parasel;

                    { elfcn00, thhat00, lahat00, g, H, itct, fcount, retcode0 } 
                    = csmin_el(thhat00_old,eye(nparaest-1),crit_global,nit_global,&objfcn_thdd,&grad_thdd);
                    retcode_cum = retcode_cum + retcode0;

                    { elfcn00, lahat00, phat00, retcode00 } = objfcn_thdd(thhat00);
                    thhat00_old = thhat00;

                    lrtest = 2*Tsim*(elfcn00-elfcn0); 
                    found  = lrtest le supcritval_s;
                   
                    
                    "s-loop low - reverse" s_test~lrtest~supcritval_s;
                    pause(1e-5);
                    

                    s_test = s_test + s_step/s_n;
                    s_ind = s_ind+1;
                    
           endo;


           cis0min = s_test - s_step/s_n;
                          
           /* compute length of the s interval
           */
           cis0length = cis0max - cis0min;
           cis0cov = (0 ge cis0min) and (0 le cis0max);
           cis0ssim[j,.] = cis0cov ~ cis0length ~ cis0min ~ cis0max;
           
           "";
           "Error Codes " retcode_cum error_smin error_smax error_thhat0;
           "";
           "u interv bds" u_test_min u_test_max;
           "u interv    " ciu0min ciu0max ciu0length ciu0cov;
           "";
           "s interv 0  " cis0min cis0max cis0length cis0cov;
           "s interv 1  " cis1min cis1max cis1length cis1cov;
           "=================================================================";
           "";
           "";
                   
           j = j+1;

    endo;   

    "Block   " iblock "of" nblocks;
    
    wr = writer(fhciu0s,ciu0ssim);
    wr = writer(fhcis1s,cis1ssim);
    wr = writer(fhcis0s,cis0ssim);
 
    tend = date;
    "Elapsed Time"; etstr(ethsec(tstart,tend));
    pause(1e-5);

    iblock = iblock+1;
endo;  

closeall fhciu0s, fhcis1s, fhcis0s;    

/** close log file **/
output file = ^outstr off;
      
end;

#include mc_procel.src
#include mc_proceldd.src
#include mc_procel1.src
#include mc_procel12.src
#include mc_procopt.src
#include critval.src
