/* filename:    paramom.g
** description: Compute MC mean and std 
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

#include nkmp_spec.g
#include pathspec.g;

mcrun  = "2";
nblock = 10000;

nparaout  = nparaest+hh+1;

file_names  =  "0pa" $|
               "1pa" $|
               "12pa";

estim_name  =  { sEL1 sEL2 sEL3 la1 la2 la3 la4 el };

lpath  = mcpath $+ "\\mcrun" $+ mcrun;
opath  = mcpath $+ "\\mcrun" $+ mcrun;

i = 1;
do until i > rows(file_names);

   file_name = file_names[i];
   draw_name = estim_name;
   draw_vecn = nparaout;
    
   lpara  = lpath $+ "\\" $+ file_names[i];
   oparas = opath $+ "\\" $+ file_names[i] $+ "s" ;

   /* locate 1,1; */
   "File " i "out of " rows(file_names);

   gosub runproc;

   i = i+1;

endo;

end;

/* SUBROUTINE
*/

runproc:

open fhdraw = ^lpara for read;

drawrow = readr( fhdraw, 1); 
drawdim = cols(drawrow);
drawrow = seekr( fhdraw, 1);

/* Part 1: Compute the mean of x(i) and x^2(i)
*/
drawmean   = zeros(1,drawdim);
drawsqmean = zeros(1,drawdim);
drawmax    = -1E10*ones(1,drawdim);
drawmin    = 1E10*ones(1,drawdim);

eofloop = 0;
ndraws  = 0;

do until eofloop; 

   drawblock = readr( fhdraw, nblock );

   drawmean   = drawmean   + sumc(drawblock)';
   drawsqmean = drawsqmean + sumc(drawblock^2)';
   drawmax    = maxc( drawmax | maxc(drawblock)' )';
   drawmin    = minc( drawmin | minc(drawblock)' )';

   ndraws = ndraws + rows(drawblock);

   /*
   locate 3,1; 
   "Draws processed:" ndraws;
   */

   eofloop = eof(fhdraw);

endo;

drawmean   = drawmean/ndraws;
drawsqmean = drawsqmean/ndraws;
drawstdd   = sqrt(drawsqmean - (drawmean)^2);

clear drawblock;

drawmae     = zeros(3,drawdim);
med_ind     = floor(ndraws/2);
quart_ind_l = floor(0.05*ndraws);
quart_ind_h = floor(0.95*ndraws);

j = 1;
do until j > drawdim;

   /* Read only the j'th column
   */
   drawrow   = seekr( fhdraw, 1); 
   drawblock = readr( fhdraw, 1);

   drawcol = drawblock[1,j];
   eofloop = 0;
   ndraws  = 1;
   
   do until eofloop; 

      drawblock = readr( fhdraw, nblock );
      drawcol   = drawcol | drawblock[.,j];
      ndraws    = ndraws + rows(drawblock);
      eofloop   = eof(fhdraw);
         
   endo;
   
   drawcol=sortc(drawcol,1);
   
   drawmae[1,j] = drawcol[med_ind];
   drawmae[2,j] = drawcol[quart_ind_h] - drawcol[quart_ind_l];
   /* drawmae[3,j] = meanc(abs(drawcol)); */
   drawcol2 = drawcol^2;
   drawcol2 = sortc(drawcol2,1);
   drawmae[3,j] = drawcol2[med_ind];


   j = j+1;

endo;
clear drawcol;


/*
goto noprint;
*/

/* Report Monte Carlo Mean and Stderror
**
*/
"DGP         " dgpname;
"MC-Run      " mcrun;
"Filename    " file_name;

"=============================================";
" " ;

Tsim_ind = 1;
do until Tsim_ind > Tsim_n;

      paraind = Tsim_ind;

      "Tsim"  Tsim_grid[Tsim_ind];
      "";

      "Statistic | Mean | StdD | MSE | Med | Range | Med(SE) ";

      outmat = draw_name' ~ drawmean[(paraind-1)*draw_vecn+1:paraind*draw_vecn]' 
                          ~ drawstdd[(paraind-1)*draw_vecn+1:paraind*draw_vecn]' 
                          ~ drawsqmean[(paraind-1)*draw_vecn+1:paraind*draw_vecn]'
                          ~ drawmae[.,(paraind-1)*draw_vecn+1:paraind*draw_vecn]' 
                          ;   

      let mask[1,7] = 0 1 1 1 1 1 1;
      let fmt[7,3]  =
      "-*.*s " 8 4
      "*.*lf " 10 3
      "*.*lf " 10 3
      "*.*lf " 10 3
      "*.*lf " 10 3
      "*.*lf " 10 3
      "*.*lf " 10 3;

      d = printfm(outmat,mask,fmt);

      "";
      "--------------------------------------------";
      "";
     
   Tsim_ind = Tsim_ind + 1;
endo;   

noprint:

/* Write Output to DISK */

create fhdraws = ^oparas with DRAWSUM, drawdim, 8;
wr = writer(fhdraws, real(drawmean | drawstdd | drawsqmean | drawmae ) );
closeall fhdraw, fhdraws; 

return;
