/* filename: mc_sim.g
** created:  06/18/03
** Modifications:
**
*/

new;
cls;
library pgraph, user;
_fcmptol = 1E-10;

#include nkmp_spec.g
#include pathspec.g;
#include nkmp_mod.src
#include nkmp_para.src
#include nkmp_sim.src

declare matrix gmat_global, lasel_global;


/******************************************************** 
** Configuration of Monte Carlo
*/
/*
rndseed(1799);
*/
mcrun    = "2";
nblocks  = 1;
nsim     = 10000;
__output = 0;
nstat    = 2;

/* Load model parameters
*/
load path = ^dgppath para[] = "dgp_para.out";
para = reshape(para,rows(para)/ndgp,ndgp);
para = para[.,dgp];

th_true = para[5] | (1-para[5])*para[3] | 4*(1-para[5])*para[4];

lpath          = resupath;
ljlim          = lpath $+ "\\jlim";
lqlim          = lpath $+ "\\qlim";
lnu            = lpath $+ "\\nu";

open fhjlim    = ^ljlim for read;
open fhqlim    = ^lqlim for read;
open fhnu      = ^lnu for read;

JJ  = readr( fhjlim, hh); 
QQ  = readr( fhqlim, npara);
nu  = readr( fhnu,hh);
nu  = nu[h1+1:hh];


/* Starting values for hessian
*/
H0  = eye(nparaest);

/******************************************************** 
** Initialize Output files
*/

nparaout  = nparaest+hh+1;
opath     = mcpath $+ "\\mcrun" $+ mcrun;
chkdir = chk_dir(opath);

opara0  = opath $+ "\\0pa";
opara1  = opath $+ "\\1pa";
opara12 = opath $+ "\\12pa";

ostat0  = opath $+ "\\0st";
ostat1  = opath $+ "\\1st";
ostat12 = opath $+ "\\12st";

create fhpara0 =^opara0  with PARA, Tsim_n*nparaout, 8;
create fhpara1 =^opara1  with PARA, Tsim_n*nparaout, 8;
create fhpara12=^opara12 with PARA, Tsim_n*nparaout, 8;

create fhstat0 =^ostat0  with STAT, Tsim_n*nstat, 8;
create fhstat1 =^ostat1  with STAT, Tsim_n*nstat, 8;
create fhstat12=^ostat12 with STAT, Tsim_n*nstat, 8;

/** start log file **/
outstr = opath $+ "\\mc_sim.log";
output file = ^outstr reset;


/******************************************************** 
** Monte Carlo Iterations
*/

iblock = 1;
do until iblock > nblocks;

   tstart = date;

   para0sim  = zeros(nsim,Tsim_n*nparaout);
   para1sim  = zeros(nsim,Tsim_n*nparaout);
   para12sim = zeros(nsim,Tsim_n*nparaout);

   stat0sim  = zeros(nsim,Tsim_n*nstat);
   stat1sim  = zeros(nsim,Tsim_n*nstat);
   stat12sim = zeros(nsim,Tsim_n*nstat);

   stat0cumerror = zeros(nsim,1);
   stat1cumerror = zeros(nsim,1);
   stat12cumerror = zeros(nsim,1);
   
   j = 1;
   
   do until j > nsim;

    valid = 0;
    do until valid == 1; 

     /* Generate Data
     */             
     yt = dsgesim(para,msel,Tsimburn,Tsimtotal);
     yt = yt - meanc(yt)';
     /* yt = Ygr, pi, R */

     /* Iterate over sample size
     */
     Tsim_ind = 1;
     
     do until Tsim_ind > Tsim_n;
     
           paraind = Tsim_ind;

           Tsim = Tsim_grid[Tsim_ind];      
           
           XY = yt[2:Tsim,3]; 
           /* R */
           
           XX = yt[1:Tsim-1,3] ~ yt[2:Tsim,2] ~ yt[2:Tsim,1];
           /* R(t-1), pi(t), Y(t) */
           
           X1 = yt[1:Tsim-1,3] ~ yt[1:Tsim-1,2] ~ yt[1:Tsim-1,1];
           /* R(t-1), pi(t-1), Y(t-1) */
           
           X2 = -yt[2:Tsim,1] ;
           /* -Y(t) */

           X12 = X1~X2;

           if (j==1) and (Tsim_ind==1) and (iblock==1);

             sighat = sqrt((XY - XX*th_true)'*(XY - XX*th_true)/Tsim);
             nuhat  = (XY - XX*th_true)'*X2/Tsim;
             thhat1 = (X1'*XY)/(X1'*XX);
  
             Qhat = -XX'*X12/Tsim;
             Jhat = X12'*X12/Tsim;
             Jhat = Jhat*sighat^2; 

             "Verification";
             "QQ / Qhat";
             QQ; Qhat;
             "JJ / Jhat";
             JJ; Jhat;
             "nu / nuhat";
             nu; nuhat;
             "sighat";
             sighat;
             "Estimated / True Parameters";
             thhat1'; th_true';
             "";
             "Press key to continue";
             kpress = keyw;
 
           endif;
              
           Xmat = XY ~ XX ~ X1 ~ X2;
                                     
           /* Version 1: use only equality conditions 
           */
           thhat1 = (X1'*XY)/(X1'*XX);
           { elfcn1, lahat1, phat1, retcode1 } = objfcn_th1(thhat1);
          
           lahat1  = lahat1  | 0;
   
           para1sim[j,(paraind-1)*nparaout+1:paraind*nparaout] 
              = ( sqrt(Tsim)*(thhat1-th_true)'    )~
                ( sqrt(Tsim)*lahat1'              )~
                Tsim*elfcn1;

           stat1sim[j,(paraind-1)*nstat+1:paraind*nstat] 
              = (retcode1 > 0) ~ sumc(phat1);  

           stat1cumerror[j,1] = stat1cumerror[j,1]+(retcode1>0);   

           /* Version 12: treat both sets of conditions as equalities
           */
           /*
           thhat_gmm12 = inv(XX'*X12*inv(X12'*X12)*X12'*XX)*XX'*X12*inv(X12'*X12)*X12'*XY;
           */
           
           x12x12x12xy = (X12'*XY)/(X12'*X12);
           x12x12x12xx = (X12'*XX)/(X12'*X12);
           thhat_gmm12 = (XX'*X12*x12x12x12xy)/(XX'*X12*x12x12x12xx);
 
           nu_global = 0;
           
           { elfcn12, thhat12, lahat12, g, H, itct, fcount, retcode12 } 
           = csmin_el(th_true,H0,crit_global,nit_global,&objfcn_th12,&grad_th12);

           if retcode12 > 0;
             "Th_true starting value failed - try thhat1";
             retcode12;
             thhat12';
             { elfcn12, thhat12, lahat12, g, H, itct, fcount, retcode12 } 
             = csmin_el(thhat1,H0,crit_global,nit_global,&objfcn_th12,&grad_th12);
             thhat12';
             thhat1';
             "";
             pause(1e-5);
           endif;

           if retcode12 > 0;
             "thhat1 starting value failed - try thhat_gmm";
             retcode12;
             thhat12';
             { elfcn12, thhat12, lahat12, g, H, itct, fcount, retcode12 } 
             = csmin_el(thhat_gmm12,H0,crit_global,nit_global,&objfcn_th12,&grad_th12);
             thhat12';
             thhat1';
             pause(1e-5);
           endif;


           stat12cumerror[j,1] = stat12cumerror[j,1]+(retcode12>0);   

           { elfcn12, lahat12, phat12, retcode12 } = objfcn_th12(thhat12);
   
           para12sim[j,(paraind-1)*nparaout+1:paraind*nparaout] 
              = ( sqrt(Tsim)*(thhat12-th_true)'    )~
                ( sqrt(Tsim)*lahat12'              )~
                Tsim*elfcn12;

           stat12sim[j,(paraind-1)*nstat+1:paraind*nstat] 
              = (retcode12 > 0) ~ sumc(phat12);  

           stat12cumerror[j,1] = stat12cumerror[j,1]+(retcode12>0);   
           
           /* Version 0: use equality and inequality conditions 
           */
                      
           if lahat12[4] ge 0;
              
               para0sim[j,(paraind-1)*nparaout+1:paraind*nparaout] 
                 = para1sim[j,(paraind-1)*nparaout+1:paraind*nparaout];

               stat0sim[j,(paraind-1)*nstat+1:paraind*nstat] 
                 = stat1sim[j,(paraind-1)*nstat+1:paraind*nstat];

               stat0cumerror[j,1] = stat1cumerror[j,1];   

           else;

               para0sim[j,(paraind-1)*nparaout+1:paraind*nparaout] 
                 = para12sim[j,(paraind-1)*nparaout+1:paraind*nparaout];

               stat0sim[j,(paraind-1)*nstat+1:paraind*nstat] 
                 = stat12sim[j,(paraind-1)*nstat+1:paraind*nstat];

               stat0cumerror[j,1] = stat12cumerror[j,1];   

           endif;
           
           endofloop:
 
           Tsim_ind = Tsim_ind+1;

     endo;

     /*
     "--------------------------------------------------------";
     "Block   " iblock "of" nblocks;
     "Element " j      "of" nsim;
     "";
     "";
     
     
     "Theta_hat  - Version 0 " para0sim[j,1:nparaest];
     "Theta_hat  - Version 1 " para1sim[j,1:nparaest];
     "Theta_hat  - Version 12" para12sim[j,1:nparaest];
     "";
     pause(1e-5);
     */

     if (stat0cumerror[j,1]==0) and (stat1cumerror[j,1]==0) and (stat12cumerror[j,1]==0);
         valid = 1;
     else;
        "Invalid Computation";
        pause(1e-5);
        stat0cumerror[j,1]=0;
        stat1cumerror[j,1]=0;
        stat12cumerror[j,1]=0;
     endif;

    endo;
    "Iteration " j
    "Value used" sqrt(Tsim)*(thhat12-th_true)';

    j = j+1;

    endo;   

    stat0sim  = delif(stat0sim,stat0cumerror .NE 0);
    stat1sim  = delif(stat1sim,stat1cumerror .NE 0);
    stat12sim = delif(stat12sim,stat12cumerror .NE 0);
   
    wr = writer(fhstat0,stat0sim);
    wr = writer(fhstat1,stat1sim);
    wr = writer(fhstat12,stat12sim);

    para0sim  = delif(para0sim, stat0cumerror .NE 0);
    para1sim  = delif(para1sim, stat1cumerror .NE 0);
    para12sim = delif(para12sim, stat12cumerror .NE 0);
    
    wr = writer(fhpara0,para0sim);
    wr = writer(fhpara1,para1sim);
    wr = writer(fhpara12,para12sim);

    tend = date;

    "Block   " iblock "of" nblocks;
    "Elapsed Time"; etstr(ethsec(tstart,tend));
    pause(1e-5);


    iblock = iblock+1;
endo;  

closeall fhstat0, fhstat1, fhstat12, fhpara0, fhpara1, fhpara12;

/** close log file **/
output file = ^outstr off;
      

end;

#include mc_procel.src
#include mc_procel1.src
#include mc_procel12.src
#include mc_procopt.src
