/******************************************************************************
* NKMP_AC.G
* converts the DSGE parameters into unconditional moments
*
* upadted:  03/11/2005
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_ac.src;
#include nkmp_spec.g;
#include pathspec.g;


/* Load model parameters
*/
load path = ^dgppath para[npara,ndgp] = "dgp_para.out";
para      = para[.,dgp];

/* matrices for the limit distribution
*/
{Qlim,Jlim,nu,XXlim,ZZlim,XZlim} = dsgemom(para,msel);

/* Display results
*/
"Parameter Choice:    " dgp;
"Model Specification: " msel;
"Parameters (theta):  "; para;

"Qlim";
Qlim;
"";
"Jlim";
Jlim;
"";
"E[g2]";
nu;


Q1  = Qlim[.,1:3];
J11 = Jlim[1:3,1:3];

var_th1 = inv(Q1*inv(J11)*Q1');
var_th12 = inv(Qlim*inv(Jlim)*Qlim');

"Covariance of Theta(1)";
var_th1;

"Covariance of Theta(12)";
var_th12;

/* weak instrument analysis
*/
SIGeta = XXlim - XZlim*inv(ZZlim)*XZlim';
SIGetainv = inv(SIGeta);
SIGetainvchol = chol(SIGetainv);

"Weak instrument indicator - First-stage Fstat";
SIGetainvchol*XZlim*inv(ZZlim)*XZlim'*SIGetainvchol'/3;

end;


/* Store matrices
*/
opath  = resupath;
chkdir = chk_dir(opath);

ojlim = opath $+ "\\jlim";
oqlim = opath $+ "\\qlim";
onu   = opath $+ "\\nu";

create fhjlim = ^ojlim with JLIM, cols(Jlim), 8;
create fhqlim = ^oqlim with QLIM, cols(Qlim), 8;
create fhnu   = ^onu with NU, cols(nu), 8;

wr = writer(fhjlim,jlim);
wr = writer(fhqlim,qlim);
wr = writer(fhnu,nu);

closeall fhjlim, fhqlim, fhnu;

end;

/* compute matrices for a sequence of kappas
*/

kap_min  = 0.05;
kap_max  = 500;
kap_n    = 10;
kap_grid = seqa(kap_min,(kap_max-kap_min)/(kap_n-1),kap_n);

J11eigv_sim = zeros(kap_n,3);
Q1eigv_sim  = zeros(kap_n,3);
nu_sim = zeros(kap_n,1);

kap_ind = 1;
do until kap_ind > kap_n;
  
   kap = kap_grid[kap_ind];
   para_loop = para;
   para_loop[2] = kap;
   {Qlim,Jlim,nu} = dsgemom(para_loop,msel);

   Q1  = Qlim[.,1:3];
   J11 = Jlim[1:3,1:3];
   
   nu_sim[kap_ind,1] = nu[4];
   Q1eigv_sim[kap_ind,.] = sortc(abs(eig(Q1)),1)';
   J11eigv_sim[kap_ind,.] = sortc(abs(eig(J11)),1)';
   
   kap_ind = kap_ind+1;

endo;

graphset;
begwind;
margin(0,0,0.2,0.2);
window(3,1,0);
fonts("microb");
_ptitlht = 0.3;
_paxht = 0.3;
_pnumht = 0.25;
_pltype = {6 2 1};
_pcolor = {4 1 2};
_protate = 0; 
_plctrl = 0;
_plwidth = 8.5;
_psymsiz = 5;
_protate = 1;
/* _plegstr = "EV 1\000EV 2\000EV3";
_plegctl = {1 4 0.5 8}; */

xlabel("Kappa");

title("E(g2)");
xy(kap_grid,nu_sim);

nextwind;

title("Smallest Eigenvalues of Q1");
xy(kap_grid,Q1eigv_sim[.,1]);

nextwind;

title("Smallest Eigenvalues of J11");
xy(kap_grid,J11eigv_sim[.,1]);

endwind;

end;
