/* filename:    paratab-post1.g
** description: generates a LaTeX Table  
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

#include nkmp_spec.g
#include pathspec.g;

nblock = 15;

nparaout  = nparaest+hh+1;

lmcpath    = mcpath    $+ "\\mcrun2" ;
lasymppath = asymppath $+ "\\mcrun1" ;

/******************************************************************
**         Load Parameter Draws from MH Output
*/

/* Specify input files - you have to edit the inputfile_str
** to read the desired estimates
** filename: lmodel + lprior + dataselstr + subTstr + pas
*/
inputfile_str = "\\0pas" $|
                "\\1pas" $|
                "\\12pas";
inputfile_n   = rows(inputfile_str);

/* Specify row names for table
*/
stat_name =     "Median    &  "
             $| "Range     &  "
             $| "Median(SE)&  ";
             
stat_n = rows(stat_name);
             
para_name =  "$\\rho_R$            "           
          $| "$(1-\\rho_R)\\psi_1$  "
          $| "$4(1-\\rho_R)\\psi_2$ ";

para_n = rows(para_name);          

mc_mae    = zeros(inputfile_n*stat_n,Tsim_n*nparaout);
asymp_mae = zeros(inputfile_n*stat_n,Tsim_n*nparaout);

/* read the input files
*/
inputfile_ind = 1;

do until inputfile_ind > inputfile_n;

   
   lparas = lmcpath $+ inputfile_str[inputfile_ind];
   open fhparas = ^lparas for read; 
   rowsel=seekr(fhparas,4); 
   rowread = readr(fhparas,3);
   /* rowread[3,.] = sqrt(rowread[3,.]);  */
   mc_mae[1+(inputfile_ind-1)*stat_n:stat_n*inputfile_ind,.] = rowread;  
   closeall fhparas;
   

   lparas = lasymppath $+ inputfile_str[inputfile_ind];
   open fhparas = ^lparas for read;  
   rowsel=seekr(fhparas,4); 
   rowread = readr(fhparas,3);
   /* rowread[3,.] = sqrt(rowread[3,.]); */
   asymp_mae[1+(inputfile_ind-1)*stat_n:stat_n*inputfile_ind,.] = rowread;  
   closeall fhparas;

   inputfile_ind = inputfile_ind+1;
   
endo;

/* Create table
*/

Tsim_ind = 1;
do until Tsim_ind > Tsim_n;

   stat_ind=1;
   do until stat_ind > stat_n;

      para_ind = 1;
      do until para_ind > para_n;

         if para_ind == 1;
            tablerow = stat_name[stat_ind] $+ para_name[para_ind];
         else;
            tablerow = "          &  " $+ para_name[para_ind];
         endif;
         
         est_ind = 1;
         do until est_ind > inputfile_n;
            tablerow = tablerow $+ " & " 
                    $+ ftos(asymp_mae[stat_n*(est_ind-1)+stat_ind,(Tsim_ind-1)*nparaout+para_ind],"%*.*lf",5,2);
            est_ind = est_ind + 1;
         endo;

         est_ind = 1;
         do until est_ind > inputfile_n;
            tablerow = tablerow $+ " & " 
                    $+ ftos(mc_mae[stat_n*(est_ind-1)+stat_ind,(Tsim_ind-1)*nparaout+para_ind],"%*.*lf",5,2);
            est_ind = est_ind + 1;
         endo;

         tablerow  = tablerow  $+ "\\\\";
         $tablerow;
      
         para_ind = para_ind+1;
      endo;
   
      stat_ind = stat_ind+1;   
   endo;
     
   Tsim_ind=Tsim_ind+1; 
endo;
